import { Extension, Facet, Text } from "@codemirror/state";
import { LRLanguage, LanguageSupport, LanguageDescription } from "@codemirror/language";
import { ViewPlugin, ViewUpdate, PluginValue, EditorView, Tooltip } from "@codemirror/view";
import { Diagnostic } from "@codemirror/lint";
import { CompletionContext, CompletionResult, Completion } from "@codemirror/autocomplete";
import { Hover, Position, MetaStore, CompletionItem } from "@rainprotocol/rainlang";
import { Diagnostic as lspDiagnostic } from "@rainprotocol/rainlang";
/**
 * @public Rainlang syntax highlighter
 */
declare const RainlangLR: LRLanguage;
// const useLast = (values: readonly any[]) => values.reduce((_, v) => v, '');
// const documentUri = Facet.define<string, string>({ combine: useLast });
// const languageId = Facet.define<string, string>({ combine: useLast });
/**
 * @public A plugin that manages languages services processes.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 */
declare class RainLanguageServicesPlugin implements PluginValue {
    private view;
    readonly uri = "file:///untitled.rain";
    readonly languageId = "rainlang";
    version: number;
    private textDocument;
    private langServices;
    private metaStore;
    /**
     * @public constructor of the class
     * @param view - The editor view instance
     * @param metaStore - (optional) The meta store object to use
     */
    constructor(view: EditorView, metaStore?: MetaStore);
    // update the instance of TexTdocument and RainDocument
    update(update: ViewUpdate): void;
    destroy(): void;
    /**
     * @public
     * Get the active meta store object of this plugin instance in order to manulay
     * update it for example update the meta store subgraphs or add a new k/v meta
     */
    getMetaStore(): MetaStore;
    /**
     * @public Handles calls for hover tooltip
     */
    handleHoverTooltip(view: EditorView, position: Position): Promise<Tooltip | null>;
    /**
     * @public Handles calls for code completion
     */
    handleCompletion(context: CompletionContext, position: Position): Promise<CompletionResult | null>;
    /**
     * @public Handles calls for docuement diagnostics
     */
    processDiagnostics(): Promise<void>;
}
/**
 * @public Converts LSP hover to codemirror hover tooltips.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror Text object
 * @param hover - The LSP hover item
 * @param position - Position of the textDocument to get the completion items for
 * @returns Codemirror tooltip or null
 */
declare function getHoverTooltip(doc: Text, hover: Hover, position: Position): Tooltip | null;
/**
 * @public Converts LSP CompletionItem to codemirror code completions.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param context - Completion context
 * @param completionItems - The LSP completion items
 * @returns Codemirror completion result or null
 */
declare function getCompletion(context: CompletionContext, completionItems: CompletionItem[]): CompletionResult | null;
/**
 * @public Converts LSP Diagnostics to codemirror diagnostics.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror Text object
 * @param diagnostics - The LSP Diagnostics
 */
declare function getDiagnostics(doc: Text, diagnostics: lspDiagnostic[]): Diagnostic[];
/**
 * @public Converts a position to offset of a character in a codemirror text.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror text
 * @param pos - The position to convert
 * @returns - The offset of the provided position
 */
declare function posToOffset(doc: Text, pos: {
    line: number;
    character: number;
}): number | undefined;
/**
 * @public Converts offset of a character in a codemirror text to a position.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param doc - Codemirror text
 * @param offset - The offset to convert
 * @returns - The position of the provided character offset
 */
declare function offsetToPos(doc: Text, offset: number): {
    line: number;
    character: number;
};
/**
 * @public Constructs regexp out of completion items.
 * Original code from [codemirror-languageserver](https://github.com/FurqanSoftware/codemirror-languageserver),
 * BSD-3-Clause License, Copyright (c) 2021, Mahmud Ridwan, All rights reserved.
 *
 * @see [LICENSE](./LICENSE) for license details.
 *
 * @param completions - Codemirror completion items
 * @returns Regexp of completion items
 */
declare function prefixMatch(completions: Completion[]): RegExp[];
/**
 * @public Reduces an array to its last element
 * @param values - Values to reduce
 */
declare const useLast: (values: readonly any[]) => any;
/**
 * @public The facet used to store and retrive RainLanguageServicesPlugin instance from a EditorView.plugin().
 * By getting the facet value from the state associated with that EditoreView instance, you then can intract
 * with the plugin directly to get the MetaStore instance and update its subgraph and metas manually, however,
 * this is not recommended as the meta storage and management is handled automatically by every meta hash that
 * is specified in an Rain document, so using this will not be needed in most of the cases unless some custom
 * behavior is intended.
 * This combination is provided for `rainlang` LanguageSupport class object and not for `RainlangExtention` class.
 *
 * @example
 * ```typescript
 * // get value of the facet
 * const rainViewPlugin = editorView.state.facet(RainLanguageServicesFacet);
 *
 * // get the plugin instance
 * const rainPlugin = editorView.plugin(rainViewPlugin);
 *
 * // update op meta from plugin
 * rainPlugin.getMetaStore();
 * ```
 */
declare const RainLanguageServicesFacet: Facet<ViewPlugin<RainLanguageServicesPlugin>, ViewPlugin<RainLanguageServicesPlugin>>;
/**
 * @public Options to choose language services
 */
type LanguageServicesConfig = {
    /**
     * Provides hover tooltips
     */
    hover?: boolean;
    /**
     * Provides code completion suggestions
     */
    completion?: boolean;
};
/**
 * @public Rainlang as a codemirror extension
 * @example
 * ```typescript
 * // instantiate the extension that can be directly used as codemirror extension
 * const rainlangExtension = new RainlangExtention(options);
 *
 * // instantiate the extension with initial meta data
 * const rainlangExtension = await RainlangExtension.create({services, subgraphs, metas})
 *
 * // to get the MetaStore instance of this extension instance
 * rainlangExtension.getMetaStore();
 * ```
 */
declare class RainlangExtension {
    extension: Extension[];
    private plugin;
    private hover;
    private completion;
    /**
     * @public Constructor of the RainlangCodemirror class
     * @param config - Options for language services to include
     */
    /**
     * @public Constructor of the RainlangCodemirror class
     * @param config - Options for language services to include
     */
    constructor(config?: LanguageServicesConfig, metaStore?: MetaStore);
    /**
     * @public Creates the Rainlang extension with initial meta data (subgraphs and meta hash/bytes)
     * @param config - The configuration to create the extension with
     * @returns A promise that resolves with a RainlangExtension
     */
    /**
     * @public Creates the Rainlang extension with initial meta data (subgraphs and meta hash/bytes)
     * @param config - The configuration to create the extension with
     * @returns A promise that resolves with a RainlangExtension
     */
    static create(config?: {
        /**
         * Language service to include
         */
        services?: LanguageServicesConfig;
        /**
         * Additional subgraph endpoint URLs
         */
        subgraphs?: string[];
        /**
         * Initial meta hash and meta bytes k/v pairs
         */
        metas?: {
            [hash: string]: string;
        };
    }): Promise<RainlangExtension>;
    /**
     * @public Get the MetaStore object instance in order to manualy update its subgraphs or metas
     * Using this method is not recommended as meta management is handled completely automatically
     * from specified meta hashes in a Rain document and doesn't need any manual interference unless
     * some custom behavior is intended.
     */
    /**
     * @public Get the MetaStore object instance in order to manualy update its subgraphs or metas
     * Using this method is not recommended as meta management is handled completely automatically
     * from specified meta hashes in a Rain document and doesn't need any manual interference unless
     * some custom behavior is intended.
     */
    getMetaStore(): MetaStore | undefined;
}
/**
 * @public
 * Provides Rainlang implementation for codemirror as LanguageSupport
 * This is the standard implementation of rainlang for codemirror.
 *
 * @example
 * ```typescript
 * const rainLanguageSupport = RainLanguage(config);
 * ```
 * in order to get the MetaStore instance, it should be done through getting the plugin instance
 * from the EditorView with providing it the result of getting the `RainLanguageServicesFacet`
 * facet and then calling `getMetaStore()` for it, however this is not recommended as meta management
 * is done automatically from specified meta hashes in a Rain document:
 * ```typescript
 * // get the `RainLanguageServicesFacet` value from editor view
 * const rainViewPlugin = view.state.facet(RainLanguageServicesFacet);
 *
 * // retrive the `RainLanguageServicesPlugin` instance
 * const rainPlugin = view.plugin(rainViewPlugin);
 *
 * // get the MetaStore instance
 * rainPlugin.getMetaStore()
 * ```
 *
 * @param config - Options for language services to include
 * @param metaStore - (optional) The meta store object to use
 * @returns rainlang LanguageSupport
 */
declare function RainLanguage(config?: LanguageServicesConfig, metaStore?: MetaStore): LanguageSupport;
/**
 * @public Provides LanguageDescription class object for rainlang.
 * Use `load` to instantiate rain LanguageSupport with initial meta data
 */
declare const RainlangDescription: LanguageDescription;
export { RainLanguageServicesPlugin, getHoverTooltip, getCompletion, getDiagnostics, posToOffset, offsetToPos, prefixMatch, useLast, RainlangLR, MetaStore, RainLanguageServicesFacet, LanguageServicesConfig, RainlangExtension, RainLanguage, RainlangDescription };
